package com.ywt.mg.services;

import com.ywt.mg.core.utils.Checker;
import com.ywt.mg.domain.entities.*;
import com.ywt.mg.domain.models.ConstantDef;
import com.ywt.mg.domain.ywtDrugEntities.Pharmacy;
import com.ywt.mg.domain.ywtDrugEntities.PharmacyRepository;
import com.ywt.mg.params.offline.DownloadOfflineListNewRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 下载记录相关方法
 *
 * @author daiyihua
 */
@Service
public class DownloadRecordService {

    private static Logger logger = LoggerFactory.getLogger(DownloadRecordService.class);

    @Autowired
    private DownloadRecordRepository downloadRecordRepository;

    @Autowired
    private AuthService authService;

    /**
     * 得到（重复点击情况下的下载记录）或插入一条新的下载记录
     *
     * @param id        表ID，需要生成
     * @param name      名称（格式： 公司后台-一级菜单-二级菜单）
     * @param paramMd5  下载参数的md5
     * @param paramJson 下载参数的json
     * @param paramUrl  下载参数url
     * @return 下载记录
     */
    public DownloadRecord getOrInsertDownloadRecord(int id, String name, String paramMd5, String paramUrl) {
        int pharmacyId = 0, hospitalId = 0, currentAdminId = 0;
        String pharmacyName = "", hospitalName = "", currentAdminName = "";
        MGAdmins currentAdmin = authService.getCurrentAdmin();
        if (!Checker.isNone(currentAdmin)) {
            currentAdminId = currentAdmin.getId();
            currentAdminName = currentAdmin.getAccount();
        }
        DownloadRecord record = downloadRecordRepository.findCreatingLeastOne(currentAdminId, paramMd5, paramUrl);
        if (!Checker.isNone(record)) {
            return record;
        }
        record = new DownloadRecord();
        // todo: 需要判断是否有同样的下载
        record.setId(id);
        record.setName(name);


        Pharmacy pharmacy = authService.getCurrentPharmacy();
        if (!Checker.isNone(pharmacy)) {
            pharmacyId = pharmacy.getId();
            pharmacyName = pharmacy.getName();
        }
        Hospital hospital = authService.getCurrentHospitalByCache();
        if (!Checker.isNone(hospital)) {
            hospitalId = hospital.getId();
            hospitalName = hospital.getName();
        }
        record.setPharmacyId(pharmacyId);
        record.setPharmacyName(pharmacyName);
        record.setHospitalId(hospitalId);
        record.setHospitalName(hospitalName);
        record.setAdminId(currentAdminId);
        record.setAdminName(currentAdminName);
        record.setCreateTime(new Date());
        record.setParamMd5(paramMd5);
        record.setParamUrl(paramUrl);
        record.setStatus(0);
        record.setUpdateTime(new Date());
        downloadRecordRepository.saveAndFlush(record);
        return record;
    }

}
